//////////////////////////////////////////////////////////////////////////////////
// Company: Lafayette College
// Engineer: Chris Nadovich <nadovicc@lafayette.edu>
// 
// Create Date:    16:17:22 11/26/2013 
// Design Name:    ECE 323 CPLD Board, STANDARD Configuration
// Module Name:    cpldtop 
// Project Name:   ECE 323
// Target Devices: XC2C64A
// Tool versions: ISE 14.4
// Description: Standard configuration for CPLD Board.
//   Wires the video signals through to the generic IO as
//   outputs. 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module cpldtop(
	 input clk4mhz,
    input clk1khz,
	 input gclk2,
	 
	 input [1:0] sw,
	 output [1:0] led,
	 
	 inout [33:27] io,
	 
	 input vsync,
	 input burst,
	 input field,
	 input csync,
	 
	 output gate_b,
	 output gate_w
	 
    );
	 
// 10 bit counter on 1 kHz clock for various
// blinking and timing duties
	 wire [9:0] count;   
    counter10 U_SLOWCOUNT(.clk(clk1khz), .reset(1'b0), .en(1'b1), .q(count));
		
// LED Blink frequency selected by pushbuttons
	 assign led[0]=(sw[0])?count[7]:count[9];
	 assign led[1]=(sw[1])?count[6]:count[8];
	 
// Video signal passthrough
    assign io[27]=vsync;
    assign io[28]=burst;
    assign io[29]=field;
    assign io[30]=csync;

// Video gates controlled by io xor-ed with pushbuttons
// Gates are negative logic, asserted low to allow video to pass. The
// PULLUP constraint makes io pins default high so inversion
// is needed on IO to allow default video on and positive logic 
// gates at inputs

    assign gate_b=~io[31] ^ sw[0];
    assign gate_w=~io[32] ^ sw[1];
	 
// Count csync pulses

  wire csync_d, csync_d2; // Csync delayed by 1 and 2 clocks
  wire line_start; // true on rising edge of csync
  wire [9:0] line_count;  // count of csync pulses after vsync
  
  d_flipflop U_DFF1 (.clk(clk4mhz), .reset(1'b0), .d(csync), .q(csync_d));
  d_flipflop U_DFF2 (.clk(clk4mhz), .reset(1'b0), .d(csync_d), .q(csync_d2));
  
  assign line_start = csync_d & ~csync_d2;

  counter10 U_LINECOUNT (.clk(clk4mhz), .reset(vsync), .en(line_start), .q(line_count));
  
	 
// Spare output driven high when line count is between 42 and 57 decimal
    assign io[33] = ((line_count > 10'd42) && (line_count < 10'd57));

endmodule
